import * as plugins from '../../plugins.js';

/**
 * Certificate data structure containing all necessary information
 * about a certificate
 */
export interface CertificateData {
  domain: string;
  certificate: string;
  privateKey: string;
  expiryDate: Date;
  // Optional source and renewal information for event emissions
  source?: 'static' | 'http01' | 'dns01';
  isRenewal?: boolean;
}

/**
 * Certificates pair (private and public keys)
 */
export interface Certificates {
  privateKey: string;
  publicKey: string;
}

/**
 * Certificate failure payload type
 */
export interface CertificateFailure {
  domain: string;
  error: string;
  isRenewal: boolean;
}

/**
 * Certificate expiry payload type
 */
export interface CertificateExpiring {
  domain: string;
  expiryDate: Date;
  daysRemaining: number;
}

/**
 * Domain forwarding configuration
 */
export interface ForwardConfig {
  ip: string;
  port: number;
}

/**
 * Domain-specific forwarding configuration for ACME challenges
 */
export interface DomainForwardConfig {
  domain: string;
  forwardConfig?: ForwardConfig;
  acmeForwardConfig?: ForwardConfig;
  sslRedirect?: boolean;
}

/**
 * Domain configuration options
 */
export interface DomainOptions {
  domainName: string;
  sslRedirect: boolean;   // if true redirects the request to port 443
  acmeMaintenance: boolean; // tries to always have a valid cert for this domain
  forward?: ForwardConfig; // forwards all http requests to that target
  acmeForward?: ForwardConfig; // forwards letsencrypt requests to this config
}

/**
 * Unified ACME configuration options used across proxies and handlers
 */
export interface AcmeOptions {
  accountEmail?: string;          // Email for Let's Encrypt account
  enabled?: boolean;              // Whether ACME is enabled
  port?: number;                  // Port to listen on for ACME challenges (default: 80)
  useProduction?: boolean;        // Use production environment (default: staging)
  httpsRedirectPort?: number;     // Port to redirect HTTP requests to HTTPS (default: 443)
  renewThresholdDays?: number;    // Days before expiry to renew certificates
  renewCheckIntervalHours?: number; // How often to check for renewals (in hours)
  autoRenew?: boolean;            // Whether to automatically renew certificates
  certificateStore?: string;      // Directory to store certificates
  skipConfiguredCerts?: boolean;  // Skip domains with existing certificates
  domainForwards?: DomainForwardConfig[]; // Domain-specific forwarding configs
}

// Backwards compatibility interfaces
export interface ICertificates extends Certificates {}
export interface ICertificateData extends CertificateData {}
export interface ICertificateFailure extends CertificateFailure {}
export interface ICertificateExpiring extends CertificateExpiring {}
export interface IForwardConfig extends ForwardConfig {}
export interface IDomainForwardConfig extends DomainForwardConfig {}
export interface IDomainOptions extends DomainOptions {}
export interface IAcmeOptions extends AcmeOptions {}