import * as plugins from '../../plugins.js';
import type { 
  ForwardConfig,
  DomainOptions,
  AcmeOptions
} from '../../certificate/models/certificate-types.js';

/**
 * HTTP-specific event types
 */
export enum HttpEvents {
  REQUEST_RECEIVED = 'request-received',
  REQUEST_FORWARDED = 'request-forwarded',
  REQUEST_HANDLED = 'request-handled',
  REQUEST_ERROR = 'request-error',
}

/**
 * HTTP status codes as an enum for better type safety
 */
export enum HttpStatus {
  OK = 200,
  MOVED_PERMANENTLY = 301,
  FOUND = 302,
  TEMPORARY_REDIRECT = 307,
  PERMANENT_REDIRECT = 308,
  BAD_REQUEST = 400,
  NOT_FOUND = 404,
  METHOD_NOT_ALLOWED = 405,
  INTERNAL_SERVER_ERROR = 500,
  NOT_IMPLEMENTED = 501,
  SERVICE_UNAVAILABLE = 503,
}

/**
 * Represents a domain configuration with certificate status information
 */
export interface DomainCertificate {
  options: DomainOptions;
  certObtained: boolean;
  obtainingInProgress: boolean;
  certificate?: string;
  privateKey?: string;
  expiryDate?: Date;
  lastRenewalAttempt?: Date;
}

/**
 * Base error class for HTTP-related errors
 */
export class HttpError extends Error {
  constructor(message: string) {
    super(message);
    this.name = 'HttpError';
  }
}

/**
 * Error related to certificate operations
 */
export class CertificateError extends HttpError {
  constructor(
    message: string,
    public readonly domain: string,
    public readonly isRenewal: boolean = false
  ) {
    super(`${message} for domain ${domain}${isRenewal ? ' (renewal)' : ''}`);
    this.name = 'CertificateError';
  }
}

/**
 * Error related to server operations
 */
export class ServerError extends HttpError {
  constructor(message: string, public readonly code?: string) {
    super(message);
    this.name = 'ServerError';
  }
}

/**
 * Redirect configuration for HTTP requests
 */
export interface RedirectConfig {
  source: string;           // Source path or pattern
  destination: string;      // Destination URL
  type: HttpStatus;         // Redirect status code
  preserveQuery?: boolean;  // Whether to preserve query parameters
}

/**
 * HTTP router configuration
 */
export interface RouterConfig {
  routes: Array<{
    path: string;
    handler: (req: plugins.http.IncomingMessage, res: plugins.http.ServerResponse) => void;
  }>;
  notFoundHandler?: (req: plugins.http.IncomingMessage, res: plugins.http.ServerResponse) => void;
}

// Backward compatibility interfaces
export { HttpError as Port80HandlerError };
export { CertificateError as CertError };
export type IDomainCertificate = DomainCertificate;